@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -68,6 +68,8 @@ enum
 	SF_FORMAT_XI			= 0x0F0000,		/* Fasttracker 2 Extended Instrument */
 	SF_FORMAT_HTK			= 0x100000,		/* HMM Tool Kit format */
 	SF_FORMAT_SDS			= 0x110000,		/* Midi Sample Dump Standard */
+	SF_FORMAT_AVR			= 0x120000,		/* Audio Visual Research */
+	SF_FORMAT_WAVEX			= 0x130000,		/* MS WAVE with WAVEFORMATEX */
 	
 	/* Subtypes from here on. */
 
@@ -302,7 +304,7 @@ typedef struct
 {	int basenote ;
 	int gain ;
 	int	sustain_mode ;
-	int sustain_start, sustain_end;
+	int sustain_start, sustain_end ;
 	int release_mode ;
 	int release_start, reslease_end ;
 } SF_INSTRUMENT ;
@@ -312,7 +314,7 @@ typedef struct
 enum
 {	SF_LOOP_NONE = 800,
 	SF_LOOP_FORWARD,
-	SF_LOOP_BACKWARD,
+	SF_LOOP_BACKWARD
 } ;
 
 /* Open the specified file for read, write or both. On error, this will
