@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,6 +27,13 @@
 #include <unistd.h>
 #endif
 
+#if HAVE_ALSA_ASOUNDLIB_H
+	#define ALSA_PCM_NEW_HW_PARAMS_API
+	#define ALSA_PCM_NEW_SW_PARAMS_API
+	#include <alsa/asoundlib.h>
+	#include <sys/time.h>
+#endif
+
 #if defined (__linux__)
 	#include 	<fcntl.h>
 	#include 	<sys/ioctl.h>
@@ -41,10 +48,9 @@
 	#include <sys/ioctl.h>
 	#include <sys/audioio.h>
 
-#elif (defined (_WIN32) || defined (WIN32))
+#elif (OS_IS_WIN32 == 1)
 	#include <windows.h>
 	#include <mmsystem.h>
-	#include <mmreg.h>
 
 #endif
 
@@ -57,6 +63,269 @@
 **	Linux/OSS functions for playing a sound.
 */
 
+#if HAVE_ALSA_ASOUNDLIB_H
+
+static snd_pcm_t * alsa_open (int channels, int srate) ;
+static int alsa_write_float (snd_pcm_t *alsa_dev, float *data, int frames, int channels) ;
+
+static void
+alsa_play (int argc, char *argv [])
+{	static float buffer [BUFFER_LEN] ;
+	SNDFILE *sndfile ;
+	SF_INFO sfinfo ;
+	snd_pcm_t * alsa_dev ;
+	int		k, readcount, subformat ;
+
+puts (__func__) ;
+
+	for (k = 1 ; k < argc ; k++)
+	{	memset (&sfinfo, 0, sizeof (sfinfo)) ;
+
+		printf ("Playing %s\n", argv [k]) ;
+		if (! (sndfile = sf_open (argv [k], SFM_READ, &sfinfo)))
+		{	puts (sf_strerror (NULL)) ;
+			continue ;
+			} ;
+
+		if (sfinfo.channels < 1 || sfinfo.channels > 2)
+		{	printf ("Error : channels = %d.\n", sfinfo.channels) ;
+			continue ;
+			} ;
+
+		alsa_dev = alsa_open (sfinfo.channels, sfinfo.samplerate) ;
+
+		subformat = sfinfo.format & SF_FORMAT_SUBMASK ;
+
+		if (subformat == SF_FORMAT_FLOAT || subformat == SF_FORMAT_DOUBLE)
+		{	double	scale ;
+			int 	m ;
+
+			sf_command (sndfile, SFC_CALC_SIGNAL_MAX, &scale, sizeof (scale)) ;
+			if (scale < 1e-10)
+				scale = 1.0 ;
+			else
+				scale = 32700.0 / scale ;
+
+			while ((readcount = sf_read_float (sndfile, buffer, BUFFER_LEN)))
+			{	for (m = 0 ; m < readcount ; m++)
+					buffer [m] *= scale ;
+				alsa_write_float (alsa_dev, buffer, BUFFER_LEN / sfinfo.channels, sfinfo.channels) ;
+				} ;
+			}
+		else
+		{	while ((readcount = sf_read_float (sndfile, buffer, BUFFER_LEN)))
+				alsa_write_float (alsa_dev, buffer, BUFFER_LEN / sfinfo.channels, sfinfo.channels) ;
+			} ;
+
+		snd_pcm_close (alsa_dev) ;
+
+		sf_close (sndfile) ;
+		} ;
+
+	return ;
+} /* alsa_play */
+
+static snd_pcm_t *
+alsa_open (int channels, int samplerate)
+{	const char * device = "plughw:0" ;
+	snd_pcm_t *alsa_dev ;
+	snd_pcm_hw_params_t *hw_params ;
+	snd_pcm_uframes_t buffer_size, xfer_align, start_threshold ;
+	snd_pcm_uframes_t alsa_period_size, alsa_buffer_frames ;
+	snd_pcm_sw_params_t *sw_params ;
+
+	int err ;
+
+	alsa_period_size = 512 ;
+	alsa_buffer_frames = 3 * alsa_period_size ;
+
+	if ((err = snd_pcm_open (&alsa_dev, device, SND_PCM_STREAM_PLAYBACK, 0)) < 0)
+	{	fprintf (stderr, "cannot open audio device \"%s\" (%s)\n", device, snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	snd_pcm_nonblock (alsa_dev, 0) ;
+
+	if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0)
+	{	fprintf (stderr, "cannot allocate hardware parameter structure (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_hw_params_any (alsa_dev, hw_params)) < 0)
+	{	fprintf (stderr, "cannot initialize hardware parameter structure (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_hw_params_set_access (alsa_dev, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
+	{	fprintf (stderr, "cannot set access type (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_hw_params_set_format (alsa_dev, hw_params, SND_PCM_FORMAT_FLOAT)) < 0)
+	{	fprintf (stderr, "cannot set sample format (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_hw_params_set_rate_near (alsa_dev, hw_params, &samplerate, 0)) < 0)
+	{	fprintf (stderr, "cannot set sample rate (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_hw_params_set_channels (alsa_dev, hw_params, channels)) < 0)
+	{	fprintf (stderr, "cannot set channel count (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_hw_params_set_buffer_size_near (alsa_dev, hw_params, &alsa_buffer_frames)) < 0)
+	{	fprintf (stderr, "cannot set buffer size (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+
+	if ((err = snd_pcm_hw_params_set_period_size_near (alsa_dev, hw_params, &alsa_period_size, 0)) < 0)
+	{	fprintf (stderr, "cannot set period size (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_hw_params (alsa_dev, hw_params)) < 0)
+	{	fprintf (stderr, "cannot set parameters (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	/* extra check: if we have only one period, this code won't work */
+	snd_pcm_hw_params_get_period_size (hw_params, &alsa_period_size, 0) ;
+	snd_pcm_hw_params_get_buffer_size (hw_params, &buffer_size) ;
+	if (alsa_period_size == buffer_size)
+	{	fprintf (stderr, "Can't use period equal to buffer size (%lu == %lu)", alsa_period_size, buffer_size) ;
+		return NULL ;
+		} ;
+
+	snd_pcm_hw_params_free (hw_params) ;
+
+	if ((err = snd_pcm_sw_params_malloc (&sw_params)) != 0)
+	{	fprintf (stderr, "%s: snd_pcm_sw_params_malloc: %s", __func__, snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_sw_params_current (alsa_dev, sw_params)) != 0)
+	{	fprintf (stderr, "%s: snd_pcm_sw_params_current: %s", __func__, snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	/* note: set start threshold to delay start until the ring buffer is full */
+	snd_pcm_sw_params_current (alsa_dev, sw_params) ;
+	if ((err = snd_pcm_sw_params_get_xfer_align (sw_params, &xfer_align)) < 0)
+	{	fprintf (stderr, "cannot get xfer align (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	/* round up to closest transfer boundary */
+	start_threshold = (buffer_size / xfer_align) * xfer_align ;
+	if (start_threshold < 1)
+		start_threshold = 1 ;
+	if ((err = snd_pcm_sw_params_set_start_threshold (alsa_dev, sw_params, start_threshold)) < 0)
+	{	fprintf (stderr, "cannot set start threshold (%s)\n", snd_strerror (err)) ;
+		return NULL ;
+		} ;
+
+	if ((err = snd_pcm_sw_params (alsa_dev, sw_params)) != 0)
+	{	fprintf (stderr, "%s: snd_pcm_sw_params: %s", __func__, snd_strerror (err)) ;
+		return NULL ;
+		} ;
+	snd_pcm_sw_params_free (sw_params) ;
+
+	snd_pcm_reset (alsa_dev) ;
+
+	return alsa_dev ;
+} /* alsa_open */
+
+static int
+alsa_write_float (snd_pcm_t *alsa_dev, float *data, int frames, int channels)
+{	static	int epipe_count = 0 ;
+
+	snd_pcm_status_t *status ;
+	int total = 0 ;
+	int retval ;
+
+	if (epipe_count > 0)
+		epipe_count -- ;
+
+	while (total < frames)
+	{	retval = snd_pcm_writei (alsa_dev, data + total * channels, frames - total) ;
+
+		if (retval >= 0)
+		{	total += retval ;
+			if (total == frames)
+				return total ;
+
+			continue ;
+			} ;
+
+		switch (retval)
+		{	case -EAGAIN :
+					puts ("alsa_write_float: EAGAIN") ;
+					continue ;
+					break ;
+
+			case -EPIPE :
+					if (epipe_count > 0)
+					{	printf ("alsa_write_float: EPIPE %d\n", epipe_count) ;
+						if (epipe_count > 140)
+							return retval ;
+						} ;
+					epipe_count += 100 ;
+
+					if (0)
+					{	snd_pcm_status_alloca (&status) ;
+						if ((retval = snd_pcm_status (alsa_dev, status)) < 0)
+							fprintf (stderr, "alsa_out: xrun. can't determine length\n") ;
+						else if (snd_pcm_status_get_state (status) == SND_PCM_STATE_XRUN)
+						{	struct timeval now, diff, tstamp ;
+
+							gettimeofday (&now, 0) ;
+							snd_pcm_status_get_trigger_tstamp (status, &tstamp) ;
+							timersub (&now, &tstamp, &diff) ;
+
+							fprintf (stderr, "alsa_write_float xrun: of at least %.3f msecs. resetting stream\n",
+									diff.tv_sec * 1000 + diff.tv_usec / 1000.0) ;
+							}
+						else
+							fprintf (stderr, "alsa_write_float: xrun. can't determine length\n") ;
+						} ;
+
+					snd_pcm_prepare (alsa_dev) ;
+					break ;
+
+			case -EBADFD :
+					fprintf (stderr, "alsa_write_float: Bad PCM state.n") ;
+					return 0 ;
+					break ;
+
+			case -ESTRPIPE :
+					fprintf (stderr, "alsa_write_float: Suspend event.n") ;
+					return 0 ;
+					break ;
+
+			case -EIO :
+					puts ("alsa_write_float: EIO") ;
+					return 0 ;
+
+			default :
+					fprintf (stderr, "alsa_write_float: retval = %d\n", retval) ;
+					return 0 ;
+					break ;
+			} ; /* switch */
+		} ; /* while */
+
+	return total ;
+} /* alsa_write_float */
+
+#endif /* HAVE_ALSA_ASOUNDLIB_H */
+
+/*------------------------------------------------------------------------------
+**	Linux/OSS functions for playing a sound.
+*/
+
 #if defined (__linux__)
 
 static	int	linux_open_dsp_device (int channels, int srate) ;
@@ -108,9 +377,9 @@ linux_play (int argc, char *argv [])
 				write (audio_device, buffer, readcount * sizeof (short)) ;
 			} ;
 
-		sf_close (sndfile) ;
-
 		close (audio_device) ;
+
+		sf_close (sndfile) ;
 		} ;
 
 	return ;
@@ -122,15 +391,15 @@ linux_open_dsp_device (int channels, int srate)
 
 	if ((fd = open ("/dev/dsp", O_WRONLY, 0)) == -1 &&
 		(fd = open ("/dev/sound/dsp", O_WRONLY, 0)) == -1)
-	{	perror("linux_open_dsp_device : open ") ;
+	{	perror ("linux_open_dsp_device : open ") ;
 		exit (1) ;
 		} ;
 
 	stereo = 0 ;
 	if (ioctl (fd, SNDCTL_DSP_STEREO, &stereo) == -1)
 	{ 	/* Fatal error */
-		perror("linux_open_dsp_device : stereo ") ;
-		exit (1);
+		perror ("linux_open_dsp_device : stereo ") ;
+		exit (1) ;
 		} ;
 
 	if (ioctl (fd, SNDCTL_DSP_RESET, 0))
@@ -171,10 +440,10 @@ linux_open_dsp_device (int channels, int srate)
 #if (defined (__MACH__) && defined (__APPLE__)) /* MacOSX */
 
 typedef struct
-{	AudioStreamBasicDescription		format;
+{	AudioStreamBasicDescription		format ;
 
-	UInt32 			buf_size;
-	AudioDeviceID 	device;
+	UInt32 			buf_size ;
+	AudioDeviceID 	device ;
 
 	SNDFILE 		*sndfile ;
 	SF_INFO 		sfinfo ;
@@ -202,19 +471,19 @@ macosx_audio_out_callback (AudioDeviceID device, const AudioTimeStamp* current_t
 
 	audio_data = (MacOSXAudioData*) client_data ;
 
-	size = data_out->mBuffers[0].mDataByteSize ;
-	sample_count = size / sizeof(float) ;
+	size = data_out->mBuffers [0].mDataByteSize ;
+	sample_count = size / sizeof (float) ;
 
 	buffer = (float*) data_out->mBuffers [0].mData ;
 
 	read_count = sf_read_float (audio_data->sndfile, buffer, sample_count) ;
-	
+
 	if (read_count < sample_count)
 	{	memset (&(buffer [read_count]), 0, (sample_count - read_count) * sizeof (float)) ;
 		/* Tell the main application to terminate. */
 		audio_data->done_playing = SF_TRUE ;
 		} ;
-	
+
 	return noErr ;
 } /* macosx_audio_out_callback */
 
@@ -267,7 +536,7 @@ macosx_play (int argc, char *argv [])
 
 		audio_data.format.mSampleRate = audio_data.sfinfo.samplerate ;
 		audio_data.format.mChannelsPerFrame = audio_data.sfinfo.channels ;
-		
+
 		if ((err = AudioDeviceSetProperty (audio_data.device, NULL, 0, false, kAudioDevicePropertyStreamFormat,
 					sizeof (AudioStreamBasicDescription), &(audio_data.format))) != noErr)
 		{	printf ("AudioDeviceSetProperty (kAudioDevicePropertyStreamFormat) failed.\n") ;
@@ -279,7 +548,7 @@ macosx_play (int argc, char *argv [])
 			return ;
 
 		/* Fire off the device. */
-		if ((err = AudioDeviceAddIOProc (audio_data.device, macosx_audio_out_callback, 
+		if ((err = AudioDeviceAddIOProc (audio_data.device, macosx_audio_out_callback,
 				(void *) &audio_data)) != noErr)
 		{	printf ("AudioDeviceAddIOProc failed.\n") ;
 			return ;
@@ -298,7 +567,7 @@ macosx_play (int argc, char *argv [])
 		{	printf ("AudioDeviceStop failed.\n") ;
 			return ;
 			} ;
-    
+
 		err = AudioDeviceRemoveIOProc (audio_data.device, macosx_audio_out_callback) ;
         if (err != noErr)
 		{	printf ("AudioDeviceRemoveIOProc failed.\n") ;
@@ -322,30 +591,30 @@ macosx_play (int argc, char *argv [])
 **  point to data instead of short*. It plain sucks!
 */
 
-#if (defined (_WIN32) || defined (WIN32))
+#if (OS_IS_WIN32 == 1)
 
 #define	WIN32_BUFFER_LEN	(1<<15)
 
 typedef struct
 {	HWAVEOUT		hwave ;
 	WAVEHDR			whdr [2] ;
-	
+
 	HANDLE			Event ;
-	
+
 	short			buffer [WIN32_BUFFER_LEN / sizeof (short)] ;
 	int				current, bufferlen ;
 
 	SNDFILE 		*sndfile ;
 	SF_INFO 		sfinfo ;
 
-	sf_count_t		remaining ;	
+	sf_count_t		remaining ;
 } Win32_Audio_Data ;
 
 
 static void
 win32_play_data (Win32_Audio_Data *audio_data)
 {	int thisread, readcount ;
-		
+
 	readcount = (audio_data->remaining > audio_data->bufferlen) ? audio_data->bufferlen : (int) audio_data->remaining ;
 
 	thisread = (int) sf_read_short (audio_data->sndfile, (short *) (audio_data->whdr [audio_data->current].lpData), readcount) ;
@@ -375,26 +644,30 @@ static DWORD CALLBACK
 win32_audio_out_callback (HWAVEOUT hwave, UINT msg, DWORD data, DWORD param1, DWORD param2)
 {	Win32_Audio_Data	*audio_data ;
 
-	if (! data)
+	if (data == 0)
 		return 1 ;
 
-	/* 
-	** I consider this technique of passing a pointer via an integer as 
-	** fundamentally broken but thats the way microsoft has defined the 
+	/* Preent compiler warnings. */
+	hwave = hwave ;
+	param1 = param2 ;
+
+	/*
+	** I consider this technique of passing a pointer via an integer as
+	** fundamentally broken but thats the way microsoft has defined the
 	** interface.
 	*/
 	audio_data = (Win32_Audio_Data*) data ;
-	
+
 	if (msg == MM_WOM_DONE)
 		win32_play_data (audio_data) ;
-		
+
   return 0 ;
 } /* win32_audio_out_callback */
 
 static void
 win32_play (int argc, char *argv [])
 {	Win32_Audio_Data	audio_data ;
-	
+
 	WAVEFORMATEX wf ;
 	int	k, error ;
 
@@ -408,24 +681,24 @@ win32_play (int argc, char *argv [])
 		{	puts (sf_strerror (NULL)) ;
 			continue ;
 			} ;
-			
+
 		audio_data.remaining = audio_data.sfinfo.frames ;
 		audio_data.current = 0 ;
-		
+
 		audio_data.Event = CreateEvent (0, FALSE, FALSE, 0) ;
-			
+
 		wf.nChannels = audio_data.sfinfo.channels ;
 		wf.wFormatTag = WAVE_FORMAT_PCM ;
 		wf.cbSize = 0 ;
 		wf.wBitsPerSample = 16 ;
 
 		wf.nSamplesPerSec = audio_data.sfinfo.samplerate ;
-		
+
 		wf.nBlockAlign = audio_data.sfinfo.channels * sizeof (short) ;
-		
+
 		wf.nAvgBytesPerSec = wf.nBlockAlign * wf.nSamplesPerSec ;
 
-		error = waveOutOpen (&(audio_data.hwave), WAVE_MAPPER, &wf, (DWORD) win32_audio_out_callback, 
+		error = waveOutOpen (&(audio_data.hwave), WAVE_MAPPER, &wf, (DWORD) win32_audio_out_callback,
 							(DWORD) &audio_data, CALLBACK_FUNCTION) ;
 		if (error)
 		{	puts ("waveOutOpen failed.") ;
@@ -437,22 +710,25 @@ win32_play (int argc, char *argv [])
 
 		audio_data.whdr [0].lpData = (char*) audio_data.buffer ;
 		audio_data.whdr [1].lpData = ((char*) audio_data.buffer) + sizeof (audio_data.buffer) / 2 ;
-		
+
 		audio_data.whdr [0].dwBufferLength = sizeof (audio_data.buffer) / 2 ;
 		audio_data.whdr [1].dwBufferLength = sizeof (audio_data.buffer) / 2 ;
-		
-		audio_data.bufferlen = sizeof (audio_data.buffer) / 2 / sizeof (short) ; 		
+
+		audio_data.whdr [0].dwFlags = 0 ;
+		audio_data.whdr [1].dwFlags = 0 ;
+
+		audio_data.bufferlen = sizeof (audio_data.buffer) / 2 / sizeof (short) ;
 
 		/* Prepare the WAVEHDRs */
 		if ((error = waveOutPrepareHeader (audio_data.hwave, &(audio_data.whdr [0]), sizeof (WAVEHDR))))
-		{	printf("waveOutPrepareHeader [0] failed : %08X\n", error) ;
+		{	printf ("waveOutPrepareHeader [0] failed : %08X\n", error) ;
 			waveOutClose (audio_data.hwave) ;
 			continue ;
 			} ;
 
 		if ((error = waveOutPrepareHeader (audio_data.hwave, &(audio_data.whdr [1]), sizeof (WAVEHDR))))
-		{	printf("waveOutPrepareHeader [1] failed : %08X\n", error) ;
-			waveOutUnprepareHeader (audio_data.hwave, &(audio_data.whdr [0]), sizeof(WAVEHDR)) ;
+		{	printf ("waveOutPrepareHeader [1] failed : %08X\n", error) ;
+			waveOutUnprepareHeader (audio_data.hwave, &(audio_data.whdr [0]), sizeof (WAVEHDR)) ;
 			waveOutClose (audio_data.hwave) ;
 			continue ;
 			} ;
@@ -464,15 +740,15 @@ win32_play (int argc, char *argv [])
 		win32_play_data (&audio_data) ;
 
 		/* Wait for playback to finish. My callback notifies me when all wave data has been played */
-		WaitForSingleObject (audio_data.Event, INFINITE);
+		WaitForSingleObject (audio_data.Event, INFINITE) ;
 
 		waveOutPause (audio_data.hwave) ;
-		waveOutReset (audio_data.hwave);
+		waveOutReset (audio_data.hwave) ;
 
-		waveOutUnprepareHeader (audio_data.hwave, &(audio_data.whdr [0]), sizeof(WAVEHDR)) ;
-		waveOutUnprepareHeader (audio_data.hwave, &(audio_data.whdr [1]), sizeof(WAVEHDR)) ;
+		waveOutUnprepareHeader (audio_data.hwave, &(audio_data.whdr [0]), sizeof (WAVEHDR)) ;
+		waveOutUnprepareHeader (audio_data.hwave, &(audio_data.whdr [1]), sizeof (WAVEHDR)) ;
 
-		waveOutClose (audio_data.hwave) ;  
+		waveOutClose (audio_data.hwave) ;
 		audio_data.hwave = 0 ;
 
 		sf_close (audio_data.sndfile) ;
@@ -497,7 +773,7 @@ solaris_play (int argc, char *argv [])
 	unsigned long	delay_time ;
 	long			k, start_count, output_count, write_count, read_count ;
 	int				audio_fd, error, done ;
-	
+
 	for (k = 1 ; k < argc ; k++)
 	{	printf ("Playing %s\n", argv [k]) ;
 		if (! (sndfile = sf_open (argv [k], SFM_READ, &sfinfo)))
@@ -515,25 +791,25 @@ solaris_play (int argc, char *argv [])
 		{	perror ("open (/dev/audio) failed") ;
 			return ;
 			} ;
-	
+
 		/*	Retrive standard values. */
 		AUDIO_INITINFO (&audio_info) ;
-	
+
 		audio_info.play.sample_rate = sfinfo.samplerate ;
 		audio_info.play.channels = sfinfo.channels ;
 		audio_info.play.precision = 16 ;
 		audio_info.play.encoding = AUDIO_ENCODING_LINEAR ;
 		audio_info.play.gain = AUDIO_MAX_GAIN ;
 		audio_info.play.balance = AUDIO_MID_BALANCE ;
-		
+
 		if ((error = ioctl (audio_fd, AUDIO_SETINFO, &audio_info)))
 		{	perror ("ioctl (AUDIO_SETINFO) failed") ;
 			return ;
 			} ;
-	
+
 		/* Delay time equal to 1/4 of a buffer in microseconds. */
 		delay_time = (BUFFER_LEN * 1000000) / (audio_info.play.sample_rate * 4) ;
-					
+
 		done = 0 ;
 		while (! done)
 		{	read_count = sf_read_short (sndfile, buffer, BUFFER_LEN) ;
@@ -545,7 +821,7 @@ solaris_play (int argc, char *argv [])
 
 			start_count = 0 ;
 			output_count = BUFFER_LEN * sizeof (short) ;
-	
+
 			while (output_count > 0)
 			{	/* write as much data as possible */
 				write_count = write (audio_fd, &(buffer [start_count]), output_count) ;
@@ -559,7 +835,7 @@ solaris_play (int argc, char *argv [])
 					} ;
 				} ; /* while (outpur_count > 0) */
 			} ; /* while (! done) */
-	
+
 		close (audio_fd) ;
 		} ;
 
@@ -576,9 +852,9 @@ int
 main (int argc, char *argv [])
 {
 	if (argc < 2)
-	{	
+	{
 		printf ("\nUsage : %s <input sound file>\n\n", argv [0]) ;
-#if (defined (_WIN32) || defined (WIN32))
+#if (OS_IS_WIN32 == 1)
 		printf ("This is a Unix style command line application which\n"
 				"should be run in a MSDOS box or Command Shell window.\n\n") ;
 		printf ("Sleeping for 5 seconds before exiting.\n\n") ;
@@ -594,26 +870,25 @@ main (int argc, char *argv [])
 		} ;
 
 #if defined (__linux__)
-	#if (defined (HAVE_ALSA) && HAVE_ALSA)
+	#if HAVE_ALSA_ASOUNDLIB_H
 		if (access ("/proc/asound/cards", R_OK) == 0)
 			alsa_play (argc, argv) ;
-	#else
-		linux_play (argc, argv) ;
+		else
 	#endif
+		linux_play (argc, argv) ;
 #elif (defined (__MACH__) && defined (__APPLE__))
 	macosx_play (argc, argv) ;
 #elif (defined (sun) && defined (unix))
 	solaris_play (argc, argv) ;
-#elif (defined (_WIN32) || defined (WIN32))
+#elif (OS_IS_WIN32 == 1)
 	win32_play (argc, argv) ;
 #elif defined (__BEOS__)
 	printf ("This program cannot be compiled on BeOS.\n") ;
 	printf ("Instead, compile the file sfplay_beos.cpp.\n") ;
 	return 1 ;
 #else
-	#warning "*** Playing sound not yet supported on this platform."
-	#warning "*** Please feel free to submit a patch."
-	printf ("Error : Playing sound not yet supported on this platform.\n") ;
+	puts ("*** Playing sound not yet supported on this platform.") ;
+	puts ("*** Please feel free to submit a patch.") ;
 	return 1 ;
 #endif
 
@@ -621,7 +896,7 @@ main (int argc, char *argv [])
 } /* main */
 /*
 ** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
+** The arch-tag line is a file identity tag for the GNU Arch
 ** revision control system.
 **
 ** arch-tag: 8fc4110d-6cec-4e03-91df-0f384cabedac
