@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <ctype.h>
@@ -101,8 +102,7 @@ enum
 } ;
 
 typedef struct
-{	unsigned int	type ;
-	unsigned int	size ;
+{	unsigned int	size ;
 	short			numChannels ;
 	unsigned int	numSampleFrames ;
 	short			sampleSize ;
@@ -182,7 +182,10 @@ aiff_open	(SF_PRIVATE *psf)
 			return	SFE_BAD_OPEN_FORMAT ;
 
 		if (psf->mode == SFM_WRITE && (subformat == SF_FORMAT_FLOAT || subformat == SF_FORMAT_DOUBLE))
-		{	psf->has_peak = SF_TRUE ;
+		{	psf->pchunk = calloc (1, sizeof (PEAK_CHUNK) * psf->sf.channels * sizeof (PEAK_POS)) ;
+			if (psf->pchunk == NULL)
+				return SFE_MALLOC_FAILED ;
+			psf->has_peak = SF_TRUE ;
 			psf->peak_loc = SF_PEAK_START ;
 			} ;
 
@@ -313,7 +316,7 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 	while (! done)
 	{	psf_binheader_readf (psf, "m", &marker) ;
 
-		if (psf->mode == SFM_RDWR &&  (found_chunk & HAVE_SSND))
+		if (psf->mode == SFM_RDWR && (found_chunk & HAVE_SSND))
 			return SFE_AIFF_RW_SSND_NOT_LAST ;
 
 		switch (marker)
@@ -369,35 +372,34 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					psf_binheader_readf (psf, "E4", &dword) ;
 
 					psf_log_printf (psf, "%M : %d\n", marker, dword) ;
-					if (dword > SIGNED_SIZEOF (psf->peak))
+					if (dword != SIGNED_SIZEOF (PEAK_CHUNK) + psf->sf.channels * SIGNED_SIZEOF (PEAK_POS))
 					{	psf_binheader_readf (psf, "j", dword) ;
 						psf_log_printf (psf, "*** File PEAK chunk bigger than sizeof (PEAK_CHUNK).\n") ;
 						return SFE_WAV_BAD_PEAK ;
 						} ;
-					if (dword != SIGNED_SIZEOF (psf->peak) - SIGNED_SIZEOF (psf->peak.peak) + psf->sf.channels * SIGNED_SIZEOF (PEAK_POS))
-					{	psf_binheader_readf (psf, "j", dword) ;
-						psf_log_printf (psf, "*** File PEAK chunk size doesn't fit with number of channels.\n") ;
-						return SFE_WAV_BAD_PEAK ;
-						} ;
+
+					psf->pchunk = calloc (1, sizeof (PEAK_CHUNK) * psf->sf.channels * sizeof (PEAK_POS)) ;
+					if (psf->pchunk == NULL)
+						return SFE_MALLOC_FAILED ;
 
 					/* read in rest of PEAK chunk. */
-					psf_binheader_readf (psf, "E44", &(psf->peak.version), &(psf->peak.timestamp)) ;
+					psf_binheader_readf (psf, "E44", &(psf->pchunk->version), &(psf->pchunk->timestamp)) ;
 
-					if (psf->peak.version != 1)
-						psf_log_printf (psf, "  version    : %d *** (should be version 1)\n", psf->peak.version) ;
+					if (psf->pchunk->version != 1)
+						psf_log_printf (psf, "  version    : %d *** (should be version 1)\n", psf->pchunk->version) ;
 					else
-						psf_log_printf (psf, "  version    : %d\n", psf->peak.version) ;
+						psf_log_printf (psf, "  version    : %d\n", psf->pchunk->version) ;
 
-					psf_log_printf (psf, "  time stamp : %d\n", psf->peak.timestamp) ;
+					psf_log_printf (psf, "  time stamp : %d\n", psf->pchunk->timestamp) ;
 					psf_log_printf (psf, "    Ch   Position       Value\n") ;
 
 					cptr = (char *) psf->buffer ;
 					for (dword = 0 ; dword < psf->sf.channels ; dword++)
-					{	psf_binheader_readf (psf, "Ef4", &(psf->peak.peak [dword].value),
-														&(psf->peak.peak [dword].position)) ;
+					{	psf_binheader_readf (psf, "Ef4", &(psf->pchunk->peaks [dword].value),
+														&(psf->pchunk->peaks [dword].position)) ;
 
 						LSF_SNPRINTF (cptr, sizeof (psf->buffer), "    %2d   %-12d   %g\n",
-								dword, psf->peak.peak [dword].position, psf->peak.peak [dword].value) ;
+								dword, psf->pchunk->peaks [dword].position, psf->pchunk->peaks [dword].value) ;
 						cptr [sizeof (psf->buffer) - 1] = 0 ;
 						psf_log_printf (psf, cptr) ;
 						} ;
@@ -416,12 +418,14 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 						psf->datalength = psf->filelength - psf->dataoffset ;
 						}
 					else
-					{	psf_log_printf (psf, " SSND : %u\n", SSNDsize) ;
-						psf->dataend = psf->datalength + psf->dataoffset ;
-						} ;
+						psf_log_printf (psf, " SSND : %u\n", SSNDsize) ;
 
-					psf_log_printf (psf, "  Offset     : %d\n", ssnd_fmt.offset) ;
-					psf_log_printf (psf, "  Block Size : %d\n", ssnd_fmt.blocksize) ;
+					/* Only set dataend if there really is data at the end. */
+					if (psf->datalength + psf->dataoffset < psf->filelength)
+						psf->dataend = psf->datalength + psf->dataoffset ;
+
+					psf_log_printf (psf, "  Offset     : %u\n", ssnd_fmt.offset) ;
+					psf_log_printf (psf, "  Block Size : %u\n", ssnd_fmt.blocksize) ;
 
 					found_chunk |= HAVE_SSND ;
 
@@ -708,7 +712,7 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 	psf_log_printf (psf, " COMM : %d\n", comm_fmt->size) ;
 	psf_log_printf (psf, "  Sample Rate : %d\n", tenbytefloat2int (comm_fmt->sampleRate)) ;
-	psf_log_printf (psf, "  Frames      : %d\n", comm_fmt->numSampleFrames) ;
+	psf_log_printf (psf, "  Frames      : %u%s\n", comm_fmt->numSampleFrames, (comm_fmt->numSampleFrames == 0 && psf->filelength > 100) ? " (Should not be 0)" : "") ;
 	psf_log_printf (psf, "  Channels    : %d\n", comm_fmt->numChannels) ;
 
 	/*	Found some broken 'fl32' files with comm.samplesize == 16. Fix it here. */
@@ -832,7 +836,7 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 			psf->sf.frames = psf->datalength / (psf->bytewidth * psf->sf.channels) ;
 		} ;
 
-	if (psf->mode == SFM_RDWR && psf->dataoffset > 0 && psf->datalength > 0)
+	if (psf->mode == SFM_RDWR && psf->dataoffset > 0)
 	{	/* Assuming here that the header has already been written and just
 		** needs to be corrected for new data length. That means that we
 		** only change the length fields of the FORM and SSND chunks;
@@ -846,6 +850,13 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 		psf_binheader_writef (psf, "Etm8", FORM_MARKER, psf->filelength - 8) ;
 		psf_fwrite (psf->header, psf->headindex, 1, psf) ;
 
+		/* Now write frame count field of COMM chunk header. */
+		psf->headindex = 0 ;
+		psf_fseek (psf, 22, SEEK_SET) ;
+
+		psf_binheader_writef (psf, "Et8", psf->sf.frames) ;
+		psf_fwrite (psf->header, psf->headindex, 1, psf) ;
+
 		/* Now write new SSND chunk header. */
 		psf->headindex = 0 ;
 		psf_fseek (psf, psf->dataoffset - 16, SEEK_SET) ;
@@ -853,6 +864,11 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 		psf_binheader_writef (psf, "Etm8", SSND_MARKER, psf->datalength + SIZEOF_SSND_CHUNK) ;
 		psf_fwrite (psf->header, psf->headindex, 1, psf) ;
 
+		if (current < psf->dataoffset)
+			psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
+		else if (current > 0)
+			psf_fseek (psf, current, SEEK_SET) ;
+
 		return 0 ;
 		} ;
 
@@ -1006,10 +1022,10 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 
 	if (psf->has_peak && psf->peak_loc == SF_PEAK_START)
 	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER,
-			sizeof (psf->peak) - sizeof (psf->peak.peak) + psf->sf.channels * sizeof (PEAK_POS)) ;
+			sizeof (PEAK_CHUNK) + psf->sf.channels * sizeof (PEAK_POS)) ;
 		psf_binheader_writef (psf, "E44", 1, time (NULL)) ;
 		for (k = 0 ; k < psf->sf.channels ; k++)
-			psf_binheader_writef (psf, "Ef4", psf->peak.peak [k].value, psf->peak.peak [k].position) ; /* XXXXX */
+			psf_binheader_writef (psf, "Ef4", psf->pchunk->peaks [k].value, psf->pchunk->peaks [k].position) ; /* XXXXX */
 		} ;
 
 	/* Write SSND chunk. */
@@ -1028,7 +1044,6 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 	else if (current > 0)
 		psf_fseek (psf, current, SEEK_SET) ;
 
-
 	return psf->error ;
 } /* aiff_write_header */
 
@@ -1044,10 +1059,10 @@ aiff_write_tailer (SF_PRIVATE *psf)
 
 	if (psf->has_peak && psf->peak_loc == SF_PEAK_END)
 	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER,
-			sizeof (psf->peak) - sizeof (psf->peak.peak) + psf->sf.channels * sizeof (PEAK_POS)) ;
+			sizeof (PEAK_CHUNK) + psf->sf.channels * sizeof (PEAK_POS)) ;
 		psf_binheader_writef (psf, "E44", 1, time (NULL)) ;
 		for (k = 0 ; k < psf->sf.channels ; k++)
-			psf_binheader_writef (psf, "Ef4", psf->peak.peak [k].value, psf->peak.peak [k].position) ; /* XXXXX */
+			psf_binheader_writef (psf, "Ef4", psf->pchunk->peaks [k].value, psf->pchunk->peaks [k].position) ; /* XXXXX */
 		} ;
 
 	if (psf->str_flags & SF_STR_LOCATE_END)
@@ -1202,7 +1217,7 @@ uint2tenbytefloat (unsigned int num, unsigned char *bytes)
 
 /*
 ** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
+** The arch-tag line is a file identity tag for the GNU Arch
 ** revision control system.
 **
 ** arch-tag: 7dec56ca-d6f2-48cf-863b-a72e7e17a5d9
