@@ -326,8 +326,9 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 	int		maxlen ;
 	char	*start ;
 
-	start	= (char*) psf->header + strlen ((char*) psf->header) ;
-	maxlen	= sizeof (psf->header) - strlen ((char*) psf->header) ;
+	maxlen = strlen ((char*) psf->header) ;
+	start	= ((char*) psf->header) + maxlen ;
+	maxlen	= sizeof (psf->header) - maxlen ;
 
 	va_start (argptr, format) ;
 	LSF_VSNPRINTF (start, maxlen, format, argptr) ;
@@ -676,7 +677,6 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 **	additions:
 **
 **		p   - jump a given number of position from start of file.
-**		j   - jump a given number of bytes forward in the file.
 **
 **	If format is NULL, psf_binheader_readf returns the current offset.
 */
@@ -712,15 +712,15 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 #define	GET_BE_8BYTE(ptr)	( 	((ptr) [4] << 24)	| ((ptr) [5] << 16) |	\
 								((ptr) [6] << 8)	| ((ptr) [7]) )
 #else
-#define	GET_LE_8BYTE(ptr)	( 	((ptr) [7] << 56L)	| ((ptr) [6] << 48L) |	\
-							 	((ptr) [5] << 40L)	| ((ptr) [4] << 32L) |	\
-							 	((ptr) [3] << 24L)	| ((ptr) [2] << 16L) |	\
-							 	((ptr) [1] << 8L)	| ((ptr) [0] ))
+#define	GET_LE_8BYTE(ptr)	( 	(((ptr) [7] * 1L) << 56) | (((ptr) [6] * 1L) << 48) |	\
+							 	(((ptr) [5] * 1L) << 40) | (((ptr) [4] * 1L) << 32) |	\
+							 	(((ptr) [3] * 1L) << 24) | (((ptr) [2] * 1L) << 16) |	\
+							 	(((ptr) [1] * 1L) << 8 ) | ((ptr) [0]))
 
-#define	GET_BE_8BYTE(ptr)	( 	((ptr) [0] << 56L)	| ((ptr) [1] << 48L) |	\
-							 	((ptr) [2] << 40L)	| ((ptr) [3] << 32L) |	\
-							 	((ptr) [4] << 24L)	| ((ptr) [5] << 16L) |	\
-							 	((ptr) [6] << 8L)	| ((ptr) [7] ))
+#define	GET_BE_8BYTE(ptr)	( 	(((ptr) [0] * 1L) << 56) | (((ptr) [1] * 1L) << 48) |	\
+							 	(((ptr) [2] * 1L) << 40) | (((ptr) [3] * 1L) << 32) |	\
+							 	(((ptr) [4] * 1L) << 24) | (((ptr) [5] * 1L) << 16) |	\
+							 	(((ptr) [6] * 1L) << 8 ) | ((ptr) [7]))
 
 #endif
 
@@ -731,14 +731,12 @@ header_read (SF_PRIVATE *psf, void *ptr, int bytes)
 	if (psf->headindex + bytes > SIGNED_SIZEOF (psf->header))
 	{	if (psf->headend < SIGNED_SIZEOF (psf->header))
 			psf_log_printf (psf, "Warning : Further header read would overflow buffer.\n") ;
+		psf->headend = SIGNED_SIZEOF (psf->header) ;
 
-		bytes = sizeof (psf->header) - psf->headindex ;
-		if (bytes <= 0)
-			return 0;
-		printf ("\nR sizeof (header): %d    headindex: %4u    bytes: %d\n", SIGNED_SIZEOF (psf->header), psf->headindex, bytes) ;
+		/* This is the best that we can do. */
+		return psf_fread (ptr, 1, bytes, psf) ;
 		} ;
 
-
 	if (psf->headindex + bytes > psf->headend)
 	{	count = psf_fread (psf->header + psf->headend, 1, bytes - (psf->headend - psf->headindex), psf) ;
 		if (count != bytes - (int) (psf->headend - psf->headindex))
@@ -757,6 +755,7 @@ header_read (SF_PRIVATE *psf, void *ptr, int bytes)
 static void
 header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
 {
+
 	switch (whence)
 	{	case SEEK_SET :
 			if (position > SIGNED_SIZEOF (psf->header))
@@ -770,9 +769,14 @@ header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
 			break ;
 
 		case SEEK_CUR :
-			if (psf->headindex + position < 0 || psf->headindex >= SIGNED_SIZEOF (psf->header))
+			if (psf->headindex + position < 0)
 				break ;
 
+			if (psf->headindex >= SIGNED_SIZEOF (psf->header))
+			{	psf_fseek (psf, position, whence) ;
+				return ;
+				} ;
+
 			if (psf->headindex + position <= psf->headend)
 			{	psf->headindex += position ;
 				break ;
@@ -833,9 +837,8 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 	char			*charptr ;
 	float			*floatptr ;
 	double			*doubleptr ;
-	size_t			size ;
 	char			c ;
-	int				count = 0, position ;
+	int				byte_count = 0, count ;
 
 	if (! format)
 		return psf_ftell (psf) ;
@@ -855,14 +858,14 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case 'm' :
 					intptr = va_arg (argptr, unsigned int*) ;
 					ucptr = (unsigned char*) intptr ;
-					count += header_read (psf, ucptr, sizeof (int)) ;
+					byte_count += header_read (psf, ucptr, sizeof (int)) ;
 					*intptr = GET_MARKER (ucptr) ;
 					break ;
 
 			case 'h' :
 					intptr = va_arg (argptr, unsigned int*) ;
 					ucptr = (unsigned char*) intptr ;
-					count += header_read (psf, sixteen_bytes, sizeof (sixteen_bytes)) ;
+					byte_count += header_read (psf, sixteen_bytes, sizeof (sixteen_bytes)) ;
 					{	int k ;
 						intdata = 0 ;
 						for (k = 0 ; k < 16 ; k++)
@@ -873,13 +876,13 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case '1' :
 					charptr = va_arg (argptr, char*) ;
-					count += header_read (psf, charptr, sizeof (char)) ;
+					byte_count += header_read (psf, charptr, sizeof (char)) ;
 					break ;
 
 			case '2' :
 					shortptr = va_arg (argptr, unsigned short*) ;
 					ucptr = (unsigned char*) shortptr ;
-					count += header_read (psf, ucptr, sizeof (short)) ;
+					byte_count += header_read (psf, ucptr, sizeof (short)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*shortptr = GET_BE_SHORT (ucptr) ;
 					else
@@ -888,7 +891,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case '3' :
 					intptr = va_arg (argptr, unsigned int*) ;
-					count += header_read (psf, sixteen_bytes, 3) ;
+					byte_count += header_read (psf, sixteen_bytes, 3) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*intptr = GET_BE_3BYTE (sixteen_bytes) ;
 					else
@@ -898,7 +901,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case '4' :
 					intptr = va_arg (argptr, unsigned int*) ;
 					ucptr = (unsigned char*) intptr ;
-					count += header_read (psf, ucptr, sizeof (int)) ;
+					byte_count += header_read (psf, ucptr, sizeof (int)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*intptr = GET_BE_INT (ucptr) ;
 					else
@@ -907,7 +910,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case '8' :
 					countptr = va_arg (argptr, sf_count_t*) ;
-					count += header_read (psf, sixteen_bytes, 8) ;
+					byte_count += header_read (psf, sixteen_bytes, 8) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						countdata = GET_BE_8BYTE (sixteen_bytes) ;
 					else
@@ -918,7 +921,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case 'f' : /* Float conversion */
 					floatptr = va_arg (argptr, float *) ;
 					*floatptr = 0.0 ;
-					count += header_read (psf, floatptr, sizeof (float)) ;
+					byte_count += header_read (psf, floatptr, sizeof (float)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*floatptr = float32_be_read ((unsigned char*) floatptr) ;
 					else
@@ -928,7 +931,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case 'd' : /* double conversion */
 					doubleptr = va_arg (argptr, double *) ;
 					*doubleptr = 0.0 ;
-					count += header_read (psf, doubleptr, sizeof (double)) ;
+					byte_count += header_read (psf, doubleptr, sizeof (double)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*doubleptr = double64_be_read ((unsigned char*) doubleptr) ;
 					else
@@ -950,16 +953,16 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case 'b' :
 					charptr = va_arg (argptr, char*) ;
-					size = va_arg (argptr, int) ;
-					if (size > 0)
-						count += header_read (psf, charptr, size) ;
+					count = va_arg (argptr, int) ;
+					if (count > 0)
+						byte_count += header_read (psf, charptr, count) ;
 					break ;
 
 			case 'G' :
 					charptr = va_arg (argptr, char*) ;
-					size = va_arg (argptr, int) ;
-					if (size > 0)
-						count += header_gets (psf, charptr, size) ;
+					count = va_arg (argptr, int) ;
+					if (count > 0)
+						byte_count += header_gets (psf, charptr, count) ;
 					break ;
 
 			case 'z' :
@@ -976,16 +979,16 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case 'p' :
 					/* Get the seek position first. */
-					position = va_arg (argptr, int) ;
-					header_seek (psf, position, SEEK_SET) ;
-					count = position ;
+					count = va_arg (argptr, int) ;
+					header_seek (psf, count, SEEK_SET) ;
+					byte_count = count ;
 					break ;
 
 			case 'j' :
 					/* Get the seek position first. */
-					position = va_arg (argptr, int) ;
-					header_seek (psf, position, SEEK_CUR) ;
-					count += position ;
+					count = va_arg (argptr, int) ;
+					header_seek (psf, count, SEEK_CUR) ;
+					byte_count += count ;
 					break ;
 
 			default :
@@ -997,7 +1000,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 	va_end (argptr) ;
 
-	return count ;
+	return byte_count ;
 } /* psf_binheader_readf */
 
 /*-----------------------------------------------------------------------------------------------
@@ -1172,6 +1175,7 @@ u_bitwidth_to_subformat (int bits)
 } /* bitwidth_to_subformat */
 
 #endif /* PSF_LOG_PRINTF_ONLY */
+
 /*
 ** Do not edit or modify anything in this comment block.
 ** The arch-tag line is a file identity tag for the GNU Arch
