@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -241,10 +241,14 @@ double64_init	(SF_PRIVATE *psf)
 			} ;
 		} ;
 
-	psf->filelength = psf_get_filelen (psf) ;
-	psf->datalength = (psf->dataend) ? psf->dataend - psf->dataoffset :
+	if (psf->filelength > psf->dataoffset)
+	{	psf->datalength = (psf->dataend > 0) ? psf->dataend - psf->dataoffset :
 							psf->filelength - psf->dataoffset ;
-	psf->sf.frames = psf->datalength / (psf->sf.channels * sizeof (double)) ;
+		}
+	else
+		psf->datalength = 0 ;
+
+	psf->sf.frames = psf->datalength / psf->blockwidth ;
 
 	return 0 ;
 } /* double64_init */
@@ -438,9 +442,9 @@ double64_peak_update	(SF_PRIVATE *psf, double *buffer, int count, int indx)
 				position = k ;
 				} ;
 
-		if (fmaxval > psf->peak.peak [chan].value)
-		{	psf->peak.peak [chan].value = fmaxval ;
-			psf->peak.peak [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
+		if (fmaxval > psf->pchunk->peaks [chan].value)
+		{	psf->pchunk->peaks [chan].value = fmaxval ;
+			psf->pchunk->peaks [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
 			} ;
 		} ;
 
@@ -464,8 +468,8 @@ double64_get_capability	(SF_PRIVATE *psf)
 			return DOUBLE_CAN_RW_LE ;
 
 		/* If this test is true ints and floats are compatible and big endian. */
-		if (data.i [0] == 0x3ff3c0ca && data.i [1] == 0x428c59fb &&
-			data.c [0] == 0x3f && data.c [2] == 0xc0 && data.c [4] == 0x42 && data.c [6] == 0x59)
+		if ((data.i [0] == 0x3ff3c0ca && data.i [1] == 0x428c59fb) &&
+			(data.c [0] == 0x3f && data.c [2] == 0xc0 && data.c [4] == 0x42 && data.c [6] == 0x59))
 			return DOUBLE_CAN_RW_BE ;
 		} ;
 
@@ -984,7 +988,7 @@ bd2d_write (double *buffer, int count)
 
 /*
 ** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
+** The arch-tag line is a file identity tag for the GNU Arch
 ** revision control system.
 **
 ** arch-tag: 4ee243b7-8c7a-469b-869c-e9aa0ee3b77f
