@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -240,10 +240,14 @@ float32_init	(SF_PRIVATE *psf)
 			} ;
 		} ;
 
-	psf->filelength = psf_get_filelen (psf) ;
-	psf->datalength = (psf->dataend) ? psf->dataend - psf->dataoffset :
+	if (psf->filelength > psf->dataoffset)
+	{	psf->datalength = (psf->dataend > 0) ? psf->dataend - psf->dataoffset :
 							psf->filelength - psf->dataoffset ;
-	psf->sf.frames = psf->datalength / (psf->sf.channels * sizeof (float)) ;
+		}
+	else
+		psf->datalength = 0 ;
+
+	psf->sf.frames = psf->datalength / psf->blockwidth ;
 
 	return 0 ;
 } /* float32_init */
@@ -308,7 +312,7 @@ void
 float32_le_write (float in, unsigned char *out)
 {	int		exponent, mantissa, negative = 0 ;
 
-	*((int*) out) = 0 ;
+	memset (out, 0, sizeof (int)) ;
 
 	if (in == 0.0)
 		return ;
@@ -343,7 +347,7 @@ void
 float32_be_write (float in, unsigned char *out)
 {	int		exponent, mantissa, negative = 0 ;
 
-	*((int*) out) = 0 ;
+	memset (out, 0, sizeof (int)) ;
 
 	if (in == 0.0)
 		return ;
@@ -393,9 +397,9 @@ float32_peak_update	(SF_PRIVATE *psf, float *buffer, int count, int indx)
 				position = k ;
 				} ;
 
-		if (fmaxval > psf->peak.peak [chan].value)
-		{	psf->peak.peak [chan].value = fmaxval ;
-			psf->peak.peak [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
+		if (fmaxval > psf->pchunk->peaks [chan].value)
+		{	psf->pchunk->peaks [chan].value = fmaxval ;
+			psf->pchunk->peaks [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
 			} ;
 		} ;
 
@@ -941,7 +945,7 @@ f2bf_array (float *buffer, int count)
 
 /*
 ** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
+** The arch-tag line is a file identity tag for the GNU Arch
 ** revision control system.
 **
 ** arch-tag: b6c34917-488c-4145-9648-f4371fc4c889
