@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -43,7 +43,11 @@ typedef struct
 	sf_count_t			samplecount ;
 	short			*samples ;
 	unsigned char	*block ;
-	unsigned char	dummydata [4] ; /* Dummy size */
+#if HAVE_FLEXIBLE_ARRAY
+	unsigned short	dummydata [] ; /* ISO C99 struct flexible array. */
+#else
+	unsigned short	dummydata [1] ; /* This is a hack an might not work. */
+#endif
 } MSADPCM_PRIVATE ;
 
 /*============================================================================================
@@ -137,8 +141,8 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	pms = (MSADPCM_PRIVATE*) psf->fdata ;
 	memset (pms, 0, pmssize) ;
 
-	pms->block		= (unsigned char*) pms->dummydata ;
-	pms->samples	= (short*) (pms->dummydata + blockalign) ;
+	pms->samples	= pms->dummydata ;
+	pms->block		= (unsigned char*) (pms->dummydata + psf->sf.channels * samplesperblock) ;
 
 	pms->channels	= psf->sf.channels ;
 	pms->blocksize	= blockalign ;
@@ -169,7 +173,7 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 		} ;
 
 	if (psf->mode == SFM_WRITE)
-	{	pms->samples = (short*) (pms->dummydata + blockalign) ;
+	{	pms->samples = pms->dummydata ;
 
 		pms->samplecount = 0 ;
 
