@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -125,7 +125,7 @@ nist_read_header (SF_PRIVATE *psf)
 
 	psf->sf.format = SF_FORMAT_NIST ;
 
-	psf_header = (char*) psf->header ;
+	psf_header = (char*) psf->buffer ;
 
 	if (sizeof (psf->header) <= NIST_HEADER_LENGTH)
 		return SFE_INTERNAL ;
@@ -331,7 +331,7 @@ nist_write_header (SF_PRIVATE *psf, int calc_length)
 	psf_asciiheader_printf (psf, "end_head\n") ;
 
 	/* Zero fill to dataoffset. */
-	psf_binheader_writef (psf, "z", NIST_HEADER_LENGTH - psf->headindex) ;
+	psf_binheader_writef (psf, "z", (size_t) (NIST_HEADER_LENGTH - psf->headindex)) ;
 
 	psf_fwrite (psf->header, psf->headindex, 1, psf) ;
 
