@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -64,7 +64,11 @@ typedef struct
 	sf_count_t		sample_count ;
 	int				*samples ;
 	unsigned char	*block ;
-	int				data [1] ; /* Data size fixed during malloc (). */
+#if HAVE_FLEXIBLE_ARRAY
+	int				data [] ; /* ISO C99 struct flexible array. */
+#else
+	int				data [1] ; /* This is a hack and may not work. */
+#endif
 } PAF24_PRIVATE ;
 
 /*------------------------------------------------------------------------------
@@ -316,7 +320,7 @@ paf_write_header (SF_PRIVATE *psf, int calc_length)
 		} ;
 
 	/* Zero fill to dataoffset. */
-	psf_binheader_writef (psf, "z", (int) (psf->dataoffset - psf->headindex)) ;
+	psf_binheader_writef (psf, "z", (size_t) (psf->dataoffset - psf->headindex)) ;
 
 	psf_fwrite (psf->header, psf->headindex, 1, psf) ;
 
@@ -363,8 +367,8 @@ paf24_init (SF_PRIVATE *psf)
 	memset (ppaf24, 0, paf24size) ;
 
 	ppaf24->channels	= psf->sf.channels ;
-	ppaf24->block		= (unsigned char*) ppaf24->data ;
-	ppaf24->samples		= (int*) (ppaf24->block + PAF24_BLOCK_SIZE * ppaf24->channels) ;
+	ppaf24->samples		= ppaf24->data ;
+	ppaf24->block		= (unsigned char*) (ppaf24->data + PAF24_SAMPLES_PER_BLOCK * ppaf24->channels) ;
 
 	ppaf24->blocksize = PAF24_BLOCK_SIZE * ppaf24->channels ;
 	ppaf24->samplesperblock = PAF24_SAMPLES_PER_BLOCK ;
