@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -94,6 +94,7 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_STR_NO_ADD_END		, "Error : file type does not support strings added at end of file." },
 	{	SFE_STR_BAD_STRING		, "Error : bad string." },
 	{	SFE_STR_WEIRD			, "Error : Weird string error." },
+	{	SFE_RDWR_BAD_HEADER		, "Error : Cannot open file in read/write mode due to string data in header." },
 
 	{	SFE_WAV_NO_RIFF			, "Error in WAV file. No 'RIFF' chunk marker." },
 	{	SFE_WAV_NO_WAVE			, "Error in WAV file. No 'WAVE' chunk marker." },
@@ -476,6 +477,7 @@ sf_format_check	(const SF_INFO *info)
 
 	switch (info->format & SF_FORMAT_TYPEMASK)
 	{	case SF_FORMAT_WAV :
+		case SF_FORMAT_WAVEX :
 				/* WAV is strictly little endian. */
 				if (endian == SF_ENDIAN_BIG || endian == SF_ENDIAN_CPU)
 					return 0 ;
@@ -661,6 +663,16 @@ sf_format_check	(const SF_INFO *info)
 					return 1 ;
 				break ;
 
+		case SF_FORMAT_AVR :
+				/* SDS is strictly big endian. */
+				if (endian == SF_ENDIAN_LITTLE || endian == SF_ENDIAN_CPU)
+					return 0 ;
+				if (info->channels < 1 || info->channels > 2)
+					return 0 ;
+				if (subformat == SF_FORMAT_PCM_U8 || subformat == SF_FORMAT_PCM_S8 || subformat == SF_FORMAT_PCM_16)
+					return 1 ;
+				break ;
+
 		/*-
 		case SF_FORMAT_SD2 :
 				/+* SD2 is strictly big endian. *+/
@@ -762,7 +774,7 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 			{	int format = psf->sf.format & SF_FORMAT_TYPEMASK ;
 
 				/* Only WAV and AIFF support the PEAK chunk. */
-				if (format != SF_FORMAT_WAV && format != SF_FORMAT_AIFF)
+				if (format != SF_FORMAT_WAV && format != SF_FORMAT_WAVEX && format != SF_FORMAT_AIFF)
 					return SF_FALSE ;
 
 				format = psf->sf.format & SF_FORMAT_SUBMASK ;
@@ -1578,14 +1590,14 @@ sf_write_short	(SNDFILE *sndfile, short *ptr, sf_count_t len)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count ;
 } /* sf_write_short */
 
@@ -1618,14 +1630,14 @@ sf_writef_short	(SNDFILE *sndfile, short *ptr, sf_count_t frames)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count / psf->sf.channels ;
 } /* sf_writef_short */
 
@@ -1666,14 +1678,14 @@ sf_write_int	(SNDFILE *sndfile, int *ptr, sf_count_t len)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count ;
 } /* sf_write_int */
 
@@ -1706,14 +1718,14 @@ sf_writef_int	(SNDFILE *sndfile, int *ptr, sf_count_t frames)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count / psf->sf.channels ;
 } /* sf_writef_int */
 
@@ -1754,14 +1766,14 @@ sf_write_float	(SNDFILE *sndfile, float *ptr, sf_count_t len)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count ;
 } /* sf_write_float */
 
@@ -1794,14 +1806,14 @@ sf_writef_float	(SNDFILE *sndfile, float *ptr, sf_count_t frames)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count / psf->sf.channels ;
 } /* sf_writef_float */
 
@@ -1842,14 +1854,14 @@ sf_write_double	(SNDFILE *sndfile, double *ptr, sf_count_t len)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count ;
 } /* sf_write_double */
 
@@ -1882,14 +1894,14 @@ sf_writef_double	(SNDFILE *sndfile, double *ptr, sf_count_t frames)
 
 	psf->write_current += count / psf->sf.channels ;
 
-	if (psf->write_current > psf->sf.frames)
-		psf->sf.frames = psf->write_current ;
-
 	psf->last_op = SFM_WRITE ;
 
 	if (psf->auto_header)
 		psf->write_header (psf, SF_TRUE) ;
 
+	if (psf->write_current > psf->sf.frames)
+		psf->sf.frames = psf->write_current ;
+
 	return count / psf->sf.channels ;
 } /* sf_writef_double */
 
@@ -2034,6 +2046,9 @@ guess_file_type (SF_PRIVATE *psf, const char *filename)
 	if (buffer [0] == MAKE_MARKER ('a', 'j', 'k', 'g'))
 		return 0 /*-SF_FORMAT_SHN-*/ ;
 
+	if (buffer [0] == MAKE_MARKER ('2', 'B', 'I', 'T'))
+		return SF_FORMAT_AVR ;
+
 	if (OS_IS_MACOSX && (format = macos_guess_file_type (psf, filename)) != 0)
 		return format ;
 
@@ -2136,6 +2151,9 @@ psf_close (SF_PRIVATE *psf)
 	if (psf->dither)
 		free (psf->dither) ;
 
+	if (psf->pchunk)
+		free (psf->pchunk) ;
+
 	if (psf->format_desc)
 	{	memset (psf->format_desc, 0, strlen (psf->format_desc)) ;
 		free (psf->format_desc) ;
@@ -2149,7 +2167,7 @@ psf_close (SF_PRIVATE *psf)
 
 static int
 psf_open_file (SF_PRIVATE *psf, int mode, SF_INFO *sfinfo)
-{	int		error ;
+{	int		error, format ;
 
 	if (mode != SFM_READ && mode != SFM_WRITE && mode != SFM_RDWR)
 		return SFE_BAD_OPEN_MODE ;
@@ -2272,6 +2290,7 @@ psf_open_file (SF_PRIVATE *psf, int mode, SF_INFO *sfinfo)
 	/* Call the initialisation function for the relevant file type. */
 	switch (psf->sf.format & SF_FORMAT_TYPEMASK)
 	{	case	SF_FORMAT_WAV :
+		case	SF_FORMAT_WAVEX :
 				error = wav_open (psf) ;
 				break ;
 
@@ -2363,6 +2382,11 @@ psf_open_file (SF_PRIVATE *psf, int mode, SF_INFO *sfinfo)
 		case	SF_FORMAT_REX2 :
 				error = rx2_open (psf) ;
 				break ;
+
+		case	SF_FORMAT_AVR :
+				error = avr_open (psf) ;
+				break ;
+
 		/* Lite remove end */
 
 		default :
@@ -2373,10 +2397,10 @@ psf_open_file (SF_PRIVATE *psf, int mode, SF_INFO *sfinfo)
 		return error ;
 
 	/* For now, check whether embedding is supported. */
+	format = psf->sf.format & SF_FORMAT_TYPEMASK ;
 	if (psf->fileoffset > 0 &&
-			((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_WAV) &&
-			((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_AIFF) &&
-			((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_AU)
+			(format != SF_FORMAT_WAV) && (format != SF_FORMAT_WAVEX) &&
+			(format != SF_FORMAT_AIFF) && (format != SF_FORMAT_AU)
 			)
 		return SFE_NO_EMBED_SUPPORT ;
 
