@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2003,2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -36,9 +36,9 @@ static unsigned char aifc_data [] =
 	0x43, 0x4F, 0x4D, 0x4D, /* COMM */
 	0x00, 0x00, 0x00, 0x26, /* COMM length */
 	0x00, 0x01, 0x00, 0x00, 0x00, 0xAE, 0x00, 0x10, 0x40, 0x0D, 0xAC, 0x44,
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4E, 0x4F, 0x4E, 0x45, 0x0D, 'N' , 
-	'o' , 't' , ' ' , 'c' , 'o' , 'm' , 'p' , 'r' , 'e' , 's' , 's' , 'e' , 
-	'd' , 0x00, 
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4E, 0x4F, 0x4E, 0x45, 0x0D, 'N' ,
+	'o' , 't' , ' ' , 'c' , 'o' , 'm' , 'p' , 'r' , 'e' , 's' , 's' , 'e' ,
+	'd' , 0x00,
 
 	'F' , 'V' , 'E' , 'R' , 0x00, 0x00, 0x00, 0x04, 0xA2, 0x80, 0x51, 0x40,
 
@@ -87,10 +87,10 @@ main (void)
 	print_test_name ("aiff_rw_test", filename) ;
 
 	dump_data_to_file (filename, aifc_data, sizeof (aifc_data)) ;
-	
+
 	rw_test (filename) ;
 
-    unlink (filename) ;
+	unlink (filename) ;
 
 	puts ("ok") ;
 	return 0 ;
@@ -103,7 +103,7 @@ static void
 dump_data_to_file (const char *filename, void *data, unsigned int datalen)
 {	FILE *file ;
 
-	if ((file = fopen (filename, "w")) == NULL)
+	if ((file = fopen (filename, "wb")) == NULL)
 	{	printf ("\n\nLine %d : could not open file : %s\n\n", __LINE__, filename) ;
 		exit (1) ;
 		} ;
@@ -128,7 +128,7 @@ rw_test (const char *filename)
 	{	printf ("\n\nLine %d : sf_open SFM_READ failed : %s\n\n", __LINE__, sf_strerror (NULL)) ;
 		exit (1) ;
 		} ;
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 	sf_close (file) ;
 
 	/* Now open read/write and close the file. */
@@ -136,7 +136,7 @@ rw_test (const char *filename)
 	{	printf ("\n\nLine %d : sf_open SFM_RDWR failed : %s\n\n", __LINE__, sf_strerror (NULL)) ;
 		exit (1) ;
 		} ;
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 	sf_close (file) ;
 
 	/* Open again as read only again and fill in a new SF_INFO struct.  */
@@ -145,7 +145,7 @@ rw_test (const char *filename)
 	{	printf ("\n\nLine %d : sf_open SFM_RDWR failed : %s\n\n", __LINE__, sf_strerror (NULL)) ;
 		exit (1) ;
 		} ;
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 	sf_close (file) ;
 
 	/* Now compare the two. */
@@ -162,19 +162,9 @@ rw_test (const char *filename)
 		} ;
 
 	if (sfinfo_rd.frames != sfinfo_rw.frames)
-	{
-#if 1
-		printf ("\n\nLine %d : frame count mismatch (%ld != %ld).\n\n", __LINE__,
+	{	printf ("\n\nLine %d : frame count mismatch (%ld != %ld).\n\n", __LINE__,
 			SF_COUNT_TO_LONG (sfinfo_rd.frames), SF_COUNT_TO_LONG (sfinfo_rw.frames)) ;
 		exit (1) ;
-#else
-		puts ("\n*********************************************") ;
-		printf ("\n\nLine %d : frame count mismatch (%ld != %ld).\n\n", __LINE__,
-			SF_COUNT_TO_LONG (sfinfo_rd.frames), SF_COUNT_TO_LONG (sfinfo_rw.frames)) ;
-		/*-exit (1) ;-*/
-		puts ("*********************************************\n") ;
-		sleep (20) ;
-#endif
 		} ;
 
 	return ;
@@ -182,7 +172,7 @@ rw_test (const char *filename)
 
 /*
 ** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
+** The arch-tag line is a file identity tag for the GNU Arch
 ** revision control system.
 **
 ** arch-tag: 12561248-1ad1-4ba6-941c-029f1333c080
