@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2002 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -177,7 +177,7 @@ pcm_test_bits_8 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_short_or_die (file, 0, short_in, items, __LINE__) ;
 
@@ -239,7 +239,7 @@ pcm_test_bits_8 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_int_or_die (file, 0, int_in, items, __LINE__) ;
 
@@ -301,7 +301,7 @@ pcm_test_bits_8 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
 
@@ -365,7 +365,7 @@ pcm_test_bits_8 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
 
@@ -379,9 +379,9 @@ pcm_test_bits_8 (const char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 	/* Lite remove end */
-
-	printf ("ok\n") ;
 	unlink (filename) ;
+
+	puts ("ok") ;
 } /* pcm_test_bits_8 */
 
 static void
@@ -444,7 +444,7 @@ pcm_test_bits_16 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_short_or_die (file, 0, short_in, items, __LINE__) ;
 
@@ -506,7 +506,7 @@ pcm_test_bits_16 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_int_or_die (file, 0, int_in, items, __LINE__) ;
 
@@ -568,7 +568,7 @@ pcm_test_bits_16 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
 
@@ -632,7 +632,7 @@ pcm_test_bits_16 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
 
@@ -646,9 +646,9 @@ pcm_test_bits_16 (const char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 	/* Lite remove end */
-
-	printf ("ok\n") ;
 	unlink (filename) ;
+
+	puts ("ok") ;
 } /* pcm_test_bits_16 */
 
 static void
@@ -711,7 +711,7 @@ pcm_test_bits_24 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_short_or_die (file, 0, short_in, items, __LINE__) ;
 
@@ -773,7 +773,7 @@ pcm_test_bits_24 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_int_or_die (file, 0, int_in, items, __LINE__) ;
 
@@ -835,7 +835,7 @@ pcm_test_bits_24 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
 
@@ -899,7 +899,7 @@ pcm_test_bits_24 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
 
@@ -913,9 +913,9 @@ pcm_test_bits_24 (const char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 	/* Lite remove end */
-
-	printf ("ok\n") ;
 	unlink (filename) ;
+
+	puts ("ok") ;
 } /* pcm_test_bits_24 */
 
 static void
@@ -978,7 +978,7 @@ pcm_test_bits_32 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_short_or_die (file, 0, short_in, items, __LINE__) ;
 
@@ -1040,7 +1040,7 @@ pcm_test_bits_32 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_int_or_die (file, 0, int_in, items, __LINE__) ;
 
@@ -1102,7 +1102,7 @@ pcm_test_bits_32 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
 
@@ -1166,7 +1166,7 @@ pcm_test_bits_32 (const char *filename, int filetype, int hash)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
 
@@ -1180,9 +1180,9 @@ pcm_test_bits_32 (const char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 	/* Lite remove end */
-
-	printf ("ok\n") ;
 	unlink (filename) ;
+
+	puts ("ok") ;
 } /* pcm_test_bits_32 */
 
 
@@ -1255,7 +1255,7 @@ pcm_test_float (const char *filename, int filetype, int hash, int replace_float)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data, items, __LINE__) ;
 
@@ -1379,7 +1379,7 @@ pcm_test_float (const char *filename, int filetype, int hash, int replace_float)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_double_or_die (file, 0, data, frames, __LINE__) ;
 	for (sign = -1, k = 0 ; k < items ; k++)
@@ -1518,7 +1518,7 @@ pcm_test_double (const char *filename, int	filetype, int hash, int replace_float
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data, items, __LINE__) ;
 
@@ -1652,7 +1652,7 @@ pcm_test_double (const char *filename, int	filetype, int hash, int replace_float
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_double_or_die (file, 0, data, frames, __LINE__) ;
 
