@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2002 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -151,7 +151,7 @@ flt_scale_clip_test_16 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_float_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
@@ -281,7 +281,7 @@ flt_scale_clip_test_24 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_float_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
@@ -411,7 +411,7 @@ flt_scale_clip_test_32 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_float_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
@@ -541,7 +541,7 @@ flt_scale_clip_test_08 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_float_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_FLOAT, NULL, SF_FALSE) ;
@@ -673,7 +673,7 @@ dbl_scale_clip_test_16 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
@@ -803,7 +803,7 @@ dbl_scale_clip_test_24 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
@@ -933,7 +933,7 @@ dbl_scale_clip_test_32 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
@@ -1063,7 +1063,7 @@ dbl_scale_clip_test_08 (const char *filename, int filetype, float maxval)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data_in, HALF_BUFFER_SIZE, __LINE__) ;
 	sf_command (file, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
