@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,7 +38,9 @@
 #include <sf_unistd.h>
 #endif
 
+#include <errno.h>
 #include <string.h>
+#include <ctype.h>
 #include <math.h>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -51,7 +53,8 @@
 #define	M_PI		3.14159265358979323846264338
 #endif
 
-#define		LOG_BUFFER_SIZE		2048
+#define SIGNED_SIZEOF(x)	((int) (sizeof (x)))
+#define	LOG_BUFFER_SIZE		2048
 
 void
 gen_windowed_sine (double *data, int len, double maximum)
@@ -88,7 +91,7 @@ check_file_hash_or_die (const char *filename, unsigned int target_hash, int line
 
 	/* The 'b' in the mode string means binary for Win32. */
 	if (! (file = fopen (filename, "rb")))
-	{	printf ("\n\nLine %d: could not open file '%s'\n", line_num, filename) ;
+	{	printf ("\n\nLine %d: could not open file '%s'\n\n", line_num, filename) ;
 		exit (1) ;
 		} ;
 
@@ -111,7 +114,7 @@ check_file_hash_or_die (const char *filename, unsigned int target_hash, int line
 		} ;
 
 	if (hash1 != target_hash)
-	{	printf ("\n\nLine %d: incorrect hash value 0x%08x should be 0x%08x\n", line_num, hash1, target_hash) ;
+	{	printf ("\n\nLine %d: incorrect hash value 0x%08x should be 0x%08x\n\n", line_num, hash1, target_hash) ;
 		exit (1) ;
 		}
 
@@ -127,7 +130,7 @@ print_test_name (const char *test, const char *filename)
 		exit (1) ;
 		} ;
 
-	printf ("    %-20s : %s ", test, filename) ;
+	printf ("    %-25s : %s ", test, filename) ;
 
 	count = 24 - strlen (filename) ;
 	while (count -- > 0)
@@ -261,7 +264,7 @@ oct_save_double	(double *a, double *b, int len)
 
 
 void
-check_log_buffer_or_die (SNDFILE *file)
+check_log_buffer_or_die (SNDFILE *file, int line_num)
 {	static char	buffer [LOG_BUFFER_SIZE] ;
 	int			count ;
 
@@ -271,20 +274,20 @@ check_log_buffer_or_die (SNDFILE *file)
 	count = sf_command	(file, SFC_GET_LOG_INFO, buffer, LOG_BUFFER_SIZE) ;
 
 	if (LOG_BUFFER_SIZE - count < 2)
-	{	printf ("Possible long log buffer.\n") ;
+	{	printf ("\n\nLine %d : Possible long log buffer.\n", line_num) ;
 		exit (1) ;
 		}
 
 	/* Look for "Should" */
 	if (strstr (buffer, "ould"))
-	{	puts ("\n\nLog buffer contains `ould'. Dumping.\n") ;
+	{	printf ("\n\nLine %d : Log buffer contains `ould'. Dumping.\n", line_num) ;
 		puts (buffer) ;
 		exit (1) ;
 		} ;
 
 	/* Look for "**" */
 	if (strstr (buffer, "*"))
-	{	puts ("\n\nLog buffer contains `*'. Dumping.\n") ;
+	{	printf ("\n\nLine %d : Log buffer contains `*'. Dumping.\n", line_num) ;
 		puts (buffer) ;
 		exit (1) ;
 		} ;
@@ -311,6 +314,58 @@ string_in_log_buffer (SNDFILE *file, const char *s)
 	return strstr (buffer, s) ? SF_TRUE : SF_FALSE ;
 } /* string_in_log_buffer */
 
+void
+hexdump_file (const char * filename, sf_count_t offset, sf_count_t length)
+{
+	FILE * file ;
+	char buffer [16] ;
+	int k, m, ch, readcount ;
+
+	if (length > 1000000)
+	{	printf ("\n\nError : length (%ld) too long.\n\n", SF_COUNT_TO_LONG (offset)) ;
+		exit (1) ;
+		} ;
+
+	if ((file = fopen (filename, "r")) == NULL)
+	{	printf ("\n\nError : hexdump_file (%s) could not open file for read.\n\n", filename) ;
+		exit (1) ;
+		} ;
+
+	if (fseek (file, offset, SEEK_SET) != 0)
+	{	printf ("\n\nError : fseek(file, %ld, SEEK_SET) failed : %s\n\n", SF_COUNT_TO_LONG (offset), strerror (errno)) ;
+		exit (1) ;
+		} ;
+
+	puts ("\n\n") ;
+
+	for (k = 0 ; k < length ; k+= sizeof (buffer))
+	{	readcount = fread (buffer, 1, sizeof (buffer), file) ;
+
+		printf ("%08lx : ", SF_COUNT_TO_LONG (offset + k)) ;
+
+		for (m = 0 ; m < readcount ; m++)
+			printf ("%02x ", buffer [m] & 0xFF) ;
+
+		for (m = readcount ; m < SIGNED_SIZEOF (buffer) ; m++)
+			printf ("   ") ;
+
+		printf ("  ") ;
+		for (m = 0 ; m < readcount ; m++)
+		{	ch = isprint (buffer [m]) ? buffer [m] : '.' ;
+			putchar (ch) ;
+			} ;
+
+		if (readcount < SIGNED_SIZEOF (buffer))
+			break ;
+
+		putchar ('\n') ;
+		} ;
+
+	puts ("\n") ;
+
+	fclose (file) ;
+} /* hexdump_file */
+
 void
 dump_log_buffer (SNDFILE *file)
 {	static char	buffer [LOG_BUFFER_SIZE] ;
@@ -393,8 +448,7 @@ test_open_file_or_die (const char *filename, int mode, SF_INFO *sfinfo, int line
 		} ;
 
 	if (file == NULL)
-	{	printf ("\n\nLine %d: %s (%s) failed : ", line_num, func_name, modestr) ;
-		puts (sf_strerror (NULL)) ;
+	{	printf ("\n\nLine %d: %s (%s) failed : %s\n\n", line_num, func_name, modestr, sf_strerror (NULL)) ;
 		dump_log_buffer (file) ;
 		exit (1) ;
 		} ;
@@ -474,9 +528,9 @@ test_seek_or_die (SNDFILE *file, sf_count_t offset, int whence, sf_count_t new_p
 	channel_name = (channels == 1) ? "Mono" : "Stereo" ;
 
 	if ((position = sf_seek (file, offset, whence)) != new_pos)
-	{	printf ("Line %d : %s : sf_seek (file, %ld, %s) returned %ld.\n", line_num,
-					channel_name, SF_COUNT_TO_LONG (offset), whence_name,
-					SF_COUNT_TO_LONG (position)) ;
+	{	printf ("\n\nLine %d : %s : sf_seek (file, %ld, %s) returned %ld (should be %ld).\n\n",
+					line_num, channel_name, SF_COUNT_TO_LONG (offset), whence_name,
+					SF_COUNT_TO_LONG (position), SF_COUNT_TO_LONG (new_pos)) ;
 		exit (1) ;
 		} ;
 
