@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2002 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@ static void	pcm_test_int (const char *str, int filetype, int long_file_ok) ;
 static void	pcm_test_float (const char *str, int filetype, int long_file_ok) ;
 static void	pcm_test_double (const char *str, int filetype, int long_file_ok) ;
 
+static void empty_file_test (const char *filename, int format) ;
 
 static	double	orig_data [DATA_LENGTH] ;
 static	double	test_data [DATA_LENGTH] ;
@@ -64,6 +65,7 @@ main (int argc, char **argv)
 		printf ("           wav   - test WAV file functions (little endian)\n") ;
 		printf ("           aiff  - test AIFF file functions (big endian)\n") ;
 		printf ("           au    - test AU file functions\n") ;
+		printf ("           avr   - test AVR file functions\n") ;
 		printf ("           raw   - test RAW header-less PCM file functions\n") ;
 		printf ("           paf   - test PAF file functions\n") ;
 		printf ("           svx   - test 8SVX/16SV file functions\n") ;
@@ -78,19 +80,34 @@ main (int argc, char **argv)
 	do_all = !strcmp (argv [1], "all") ;
 
 	if (do_all || ! strcmp (argv [1], "wav"))
-	{	pcm_test_char	("char.wav"		, SF_FORMAT_WAV | SF_FORMAT_PCM_U8, SF_FALSE) ;
+	{	empty_file_test ("empty_char.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_U8) ;
+		empty_file_test ("empty_short.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16) ;
+		empty_file_test ("empty_float.wav", SF_FORMAT_WAV | SF_FORMAT_FLOAT) ;
+
+		pcm_test_char	("char.wav"		, SF_FORMAT_WAV | SF_FORMAT_PCM_U8, SF_FALSE) ;
 		pcm_test_short	("short.wav"	, SF_FORMAT_WAV | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit.wav"	, SF_FORMAT_WAV | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int.wav"		, SF_FORMAT_WAV | SF_FORMAT_PCM_32, SF_FALSE) ;
+
+		pcm_test_24bit	("24bit.wavex"	, SF_FORMAT_WAVEX | SF_FORMAT_PCM_24, SF_FALSE) ;
+		pcm_test_int	("int.wavex"	, SF_FORMAT_WAVEX | SF_FORMAT_PCM_32, SF_FALSE) ;
+
 		/* Lite remove start */
 		pcm_test_float	("float.wav"	, SF_FORMAT_WAV | SF_FORMAT_FLOAT , SF_FALSE) ;
 		pcm_test_double	("double.wav"	, SF_FORMAT_WAV | SF_FORMAT_DOUBLE, SF_FALSE) ;
+
+		pcm_test_float	("float.wavex"	, SF_FORMAT_WAVEX | SF_FORMAT_FLOAT , SF_FALSE) ;
+		pcm_test_double	("double.wavex"	, SF_FORMAT_WAVEX | SF_FORMAT_DOUBLE, SF_FALSE) ;
 		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "aiff"))
-	{	pcm_test_char	("char_u8.aiff"	, SF_FORMAT_AIFF | SF_FORMAT_PCM_U8, SF_FALSE) ;
+	{	empty_file_test ("empty_char.aiff", SF_FORMAT_AIFF | SF_FORMAT_PCM_U8) ;
+		empty_file_test ("empty_short.aiff", SF_FORMAT_AIFF | SF_FORMAT_PCM_16) ;
+		empty_file_test ("empty_float.aiff", SF_FORMAT_AIFF | SF_FORMAT_FLOAT) ;
+
+		pcm_test_char	("char_u8.aiff"	, SF_FORMAT_AIFF | SF_FORMAT_PCM_U8, SF_FALSE) ;
 		pcm_test_char	("char_s8.aiff"	, SF_FORMAT_AIFF | SF_FORMAT_PCM_S8, SF_FALSE) ;
 		pcm_test_short	("short.aiff"	, SF_FORMAT_AIFF | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit.aiff"	, SF_FORMAT_AIFF | SF_FORMAT_PCM_24, SF_FALSE) ;
@@ -168,7 +185,10 @@ main (int argc, char **argv)
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "svx"))
-	{	pcm_test_char	("char.svx" , SF_FORMAT_SVX | SF_FORMAT_PCM_S8, SF_FALSE) ;
+	{	empty_file_test ("empty_char.svx", SF_FORMAT_SVX | SF_FORMAT_PCM_S8) ;
+		empty_file_test ("empty_short.svx", SF_FORMAT_SVX | SF_FORMAT_PCM_16) ;
+
+		pcm_test_char	("char.svx" , SF_FORMAT_SVX | SF_FORMAT_PCM_S8, SF_FALSE) ;
 		pcm_test_short	("short.svx", SF_FORMAT_SVX | SF_FORMAT_PCM_16, SF_FALSE) ;
 		test_count++ ;
 		} ;
@@ -201,7 +221,10 @@ main (int argc, char **argv)
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "mat4"))
-	{	pcm_test_short	("short_be.mat4"	, SF_ENDIAN_BIG	| SF_FORMAT_MAT4 | SF_FORMAT_PCM_16, SF_FALSE) ;
+	{	empty_file_test ("empty_short.mat4", SF_FORMAT_MAT4 | SF_FORMAT_PCM_16) ;
+		empty_file_test ("empty_float.mat4", SF_FORMAT_MAT4 | SF_FORMAT_FLOAT) ;
+
+		pcm_test_short	("short_be.mat4"	, SF_ENDIAN_BIG	| SF_FORMAT_MAT4 | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_short	("short_le.mat4"	, SF_ENDIAN_LITTLE	| SF_FORMAT_MAT4 | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_int	("int_be.mat4"		, SF_ENDIAN_BIG	| SF_FORMAT_MAT4 | SF_FORMAT_PCM_32, SF_FALSE) ;
 		pcm_test_int 	("int_le.mat4"		, SF_ENDIAN_LITTLE	| SF_FORMAT_MAT4 | SF_FORMAT_PCM_32, SF_FALSE) ;
@@ -213,7 +236,11 @@ main (int argc, char **argv)
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "mat5"))
-	{	pcm_test_char 	("char_be.mat5"		, SF_ENDIAN_BIG	| SF_FORMAT_MAT5 | SF_FORMAT_PCM_U8, SF_FALSE) ;
+	{	empty_file_test ("empty_char.mat5", SF_FORMAT_MAT5 | SF_FORMAT_PCM_U8) ;
+		empty_file_test ("empty_short.mat5", SF_FORMAT_MAT5 | SF_FORMAT_PCM_16) ;
+		empty_file_test ("empty_float.mat5", SF_FORMAT_MAT5 | SF_FORMAT_FLOAT) ;
+
+		pcm_test_char 	("char_be.mat5"		, SF_ENDIAN_BIG	| SF_FORMAT_MAT5 | SF_FORMAT_PCM_U8, SF_FALSE) ;
 		pcm_test_char 	("char_le.mat5"		, SF_ENDIAN_LITTLE	| SF_FORMAT_MAT5 | SF_FORMAT_PCM_U8, SF_FALSE) ;
 		pcm_test_short	("short_be.mat5"	, SF_ENDIAN_BIG	| SF_FORMAT_MAT5 | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_short	("short_le.mat5"	, SF_ENDIAN_LITTLE	| SF_FORMAT_MAT5 | SF_FORMAT_PCM_16, SF_FALSE) ;
@@ -237,10 +264,21 @@ main (int argc, char **argv)
 	{	pcm_test_short	("short.htk", SF_FORMAT_HTK | SF_FORMAT_PCM_16, SF_FALSE) ;
 		test_count++ ;
 		} ;
+
+	if (do_all || ! strcmp (argv [1], "avr"))
+	{	pcm_test_char 	("char_u8.avr"	, SF_FORMAT_AVR | SF_FORMAT_PCM_U8, SF_FALSE) ;
+		pcm_test_char 	("char_s8.avr"	, SF_FORMAT_AVR | SF_FORMAT_PCM_S8, SF_FALSE) ;
+		pcm_test_short	("short.avr"	, SF_FORMAT_AVR | SF_FORMAT_PCM_16, SF_FALSE) ;
+		test_count++ ;
+		} ;
 	/* Lite remove end */
 
 	if (do_all || ! strcmp (argv [1], "w64"))
-	{	pcm_test_char	("char.w64"		, SF_FORMAT_W64 | SF_FORMAT_PCM_U8, SF_FALSE) ;
+	{	empty_file_test ("empty_char.w64", SF_FORMAT_W64 | SF_FORMAT_PCM_U8) ;
+		empty_file_test ("empty_short.w64", SF_FORMAT_W64 | SF_FORMAT_PCM_16) ;
+		empty_file_test ("empty_float.w64", SF_FORMAT_W64 | SF_FORMAT_FLOAT) ;
+
+		pcm_test_char	("char.w64"		, SF_FORMAT_W64 | SF_FORMAT_PCM_U8, SF_FALSE) ;
 		pcm_test_short	("short.w64"	, SF_FORMAT_W64 | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit.w64"	, SF_FORMAT_W64 | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int.w64"		, SF_FORMAT_W64 | SF_FORMAT_PCM_32, SF_FALSE) ;
@@ -351,7 +389,7 @@ pcm_test_char (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_short_or_die (file, 0, test, items, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -372,7 +410,8 @@ pcm_test_char (const char *filename, int filetype, int long_file_ok)
 
 	if ((filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_16 ||
 			(filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_24)
-	{	unlink (filename) ;
+	{	sf_close (file) ;
+		unlink (filename) ;
 		printf ("ok\n") ;
 		return ;
 		} ;
@@ -614,7 +653,7 @@ pcm_test_char (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_short_or_die (file, 0, test, frames, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -682,10 +721,9 @@ pcm_test_char (const char *filename, int filetype, int long_file_ok)
 			} ;
 
 	sf_close (file) ;
-
 	unlink (filename) ;
-	printf ("ok\n") ;
 
+	puts ("ok") ;
 	return ;
 } /* pcm_test_char */
 
@@ -756,7 +794,7 @@ pcm_test_short (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_short_or_die (file, 0, test, items, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -777,7 +815,8 @@ pcm_test_short (const char *filename, int filetype, int long_file_ok)
 
 	if ((filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_16 ||
 			(filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_24)
-	{	unlink (filename) ;
+	{	sf_close (file) ;
+		unlink (filename) ;
 		printf ("ok\n") ;
 		return ;
 		} ;
@@ -1019,7 +1058,7 @@ pcm_test_short (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_short_or_die (file, 0, test, frames, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -1087,10 +1126,9 @@ pcm_test_short (const char *filename, int filetype, int long_file_ok)
 			} ;
 
 	sf_close (file) ;
-
 	unlink (filename) ;
-	printf ("ok\n") ;
 
+	puts ("ok") ;
 	return ;
 } /* pcm_test_short */
 
@@ -1161,7 +1199,7 @@ pcm_test_24bit (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_int_or_die (file, 0, test, items, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -1182,7 +1220,8 @@ pcm_test_24bit (const char *filename, int filetype, int long_file_ok)
 
 	if ((filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_16 ||
 			(filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_24)
-	{	unlink (filename) ;
+	{	sf_close (file) ;
+		unlink (filename) ;
 		printf ("ok\n") ;
 		return ;
 		} ;
@@ -1424,7 +1463,7 @@ pcm_test_24bit (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_int_or_die (file, 0, test, frames, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -1492,10 +1531,9 @@ pcm_test_24bit (const char *filename, int filetype, int long_file_ok)
 			} ;
 
 	sf_close (file) ;
-
 	unlink (filename) ;
-	printf ("ok\n") ;
 
+	puts ("ok") ;
 	return ;
 } /* pcm_test_24bit */
 
@@ -1566,7 +1604,7 @@ pcm_test_int (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_int_or_die (file, 0, test, items, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -1587,7 +1625,8 @@ pcm_test_int (const char *filename, int filetype, int long_file_ok)
 
 	if ((filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_16 ||
 			(filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_24)
-	{	unlink (filename) ;
+	{	sf_close (file) ;
+		unlink (filename) ;
 		printf ("ok\n") ;
 		return ;
 		} ;
@@ -1829,7 +1868,7 @@ pcm_test_int (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_int_or_die (file, 0, test, frames, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -1897,10 +1936,9 @@ pcm_test_int (const char *filename, int filetype, int long_file_ok)
 			} ;
 
 	sf_close (file) ;
-
 	unlink (filename) ;
-	printf ("ok\n") ;
 
+	puts ("ok") ;
 	return ;
 } /* pcm_test_int */
 
@@ -1971,7 +2009,7 @@ pcm_test_float (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_float_or_die (file, 0, test, items, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -1992,7 +2030,8 @@ pcm_test_float (const char *filename, int filetype, int long_file_ok)
 
 	if ((filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_16 ||
 			(filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_24)
-	{	unlink (filename) ;
+	{	sf_close (file) ;
+		unlink (filename) ;
 		printf ("ok\n") ;
 		return ;
 		} ;
@@ -2234,7 +2273,7 @@ pcm_test_float (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_float_or_die (file, 0, test, frames, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -2302,10 +2341,9 @@ pcm_test_float (const char *filename, int filetype, int long_file_ok)
 			} ;
 
 	sf_close (file) ;
-
 	unlink (filename) ;
-	printf ("ok\n") ;
 
+	puts ("ok") ;
 	return ;
 } /* pcm_test_float */
 
@@ -2376,7 +2414,7 @@ pcm_test_double (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_read_double_or_die (file, 0, test, items, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -2397,7 +2435,8 @@ pcm_test_double (const char *filename, int filetype, int long_file_ok)
 
 	if ((filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_16 ||
 			(filetype & SF_FORMAT_SUBMASK) == SF_FORMAT_DWVW_24)
-	{	unlink (filename) ;
+	{	sf_close (file) ;
+		unlink (filename) ;
 		printf ("ok\n") ;
 		return ;
 		} ;
@@ -2639,7 +2678,7 @@ pcm_test_double (const char *filename, int filetype, int long_file_ok)
 		exit (1) ;
 		} ;
 
-	check_log_buffer_or_die (file) ;
+	check_log_buffer_or_die (file, __LINE__) ;
 
 	test_readf_double_or_die (file, 0, test, frames, __LINE__) ;
 	for (k = 0 ; k < items ; k++)
@@ -2707,15 +2746,79 @@ pcm_test_double (const char *filename, int filetype, int long_file_ok)
 			} ;
 
 	sf_close (file) ;
-
 	unlink (filename) ;
-	printf ("ok\n") ;
 
+	puts ("ok") ;
 	return ;
 } /* pcm_test_double */
 
 
 
+/*----------------------------------------------------------------------------------------
+*/
+
+static void
+empty_file_test (const char *filename, int format)
+{	SNDFILE		*file ;
+	SF_INFO	info ;
+
+	print_test_name ("empty_file_test", filename) ;
+
+	unlink (filename) ;
+
+	info.samplerate = 48000 ;
+	info.channels = 2 ;
+	info.format = format ;
+
+	if (sf_format_check (&info) == SF_FALSE)
+	{	info.channels = 1 ;
+		if (sf_format_check (&info) == SF_FALSE)
+		{	puts ("invalid file format") ;
+			return ;
+			} ;
+		} ;
+
+	/* Create an empty file. */
+	file = test_open_file_or_die (filename, SFM_WRITE, &info, __LINE__) ;
+	sf_close (file) ;
+
+	/* Open for read and check the length. */
+	file = test_open_file_or_die (filename, SFM_READ, &info, __LINE__) ;
+
+	if (SF_COUNT_TO_LONG (info.frames) != 0)
+	{	printf ("\n\nError : frame count (%ld) should be zero.\n", SF_COUNT_TO_LONG (info.frames)) ;
+			exit (1) ;
+			} ;
+
+	sf_close (file) ;
+
+	/* Open for read/write and check the length. */
+	file = test_open_file_or_die (filename, SFM_RDWR, &info, __LINE__) ;
+
+	if (SF_COUNT_TO_LONG (info.frames) != 0)
+	{	printf ("\n\nError : frame count (%ld) should be zero.\n", SF_COUNT_TO_LONG (info.frames)) ;
+		exit (1) ;
+		} ;
+
+	sf_close (file) ;
+
+	/* Open for read and check the length. */
+	file = test_open_file_or_die (filename, SFM_READ, &info, __LINE__) ;
+
+	if (SF_COUNT_TO_LONG (info.frames) != 0)
+	{	printf ("\n\nError : frame count (%ld) should be zero.\n", SF_COUNT_TO_LONG (info.frames)) ;
+		exit (1) ;
+		} ;
+
+	sf_close (file) ;
+
+	unlink (filename) ;
+	puts ("ok") ;
+
+	return ;
+} /* empty_file_test */
+
+
 /*----------------------------------------------------------------------------------------
 */
 
@@ -2746,6 +2849,8 @@ static int
 truncate (const char *filename, int ignored)
 {	int fd ;
 
+	ignored = 0 ;
+
 	if ((fd = open (filename, O_RDWR | O_TRUNC | O_BINARY)) < 0)
 		return 0 ;
 
